# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2010-2012, Tashilgostar co. , All Rights Reserved
#        Nima ghorbani <nima.g.67@gmail.com> (http://ir.linkedin.com/in/nimaghi)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from datetime import datetime
from dateutil.relativedelta import relativedelta
from osv import fields, osv
import base64
import random,string,os
from tools.translate import _
from render import render

class dotdict(dict):
    def __getattr__(self, attr):
        return self.get(attr, None)
    __setattr__= dict.__setitem__
    __delattr__= dict.__delitem__
class communication_queue(osv.osv):
    _name = 'comm_queue'
    _description = "Queue for Communication"
    _order = "id desc"
    def random_name(self,cr=None):
        '''Make a unique name.if cr!=None check the database for this random name and make sure it does not exist'''
        random.seed()
        if cr:
            while 1:
                res="".join([random.choice(string.ascii_letters) for x in xrange(10) ])
                cr.execute('SELECT id from comm_queue WHERE at_name=%s',(res,))
                if cr.fetchall():continue
                else:return res
        return "".join([random.choice(string.ascii_letters) for x in xrange(10) ])
    def u(self,obj, encoding='utf-8'):
        if isinstance(obj, basestring):
            if not isinstance(obj, unicode):
                obj = unicode(obj, encoding)
        return obj
    def get_suffix(self,fname):
        suffix=fname.split(u'.')
        if len(suffix)!=2:raise osv.except_osv(_('Printer Manger - Error!'), _('Should not use "." in filename %s!!!' % fname))
        return suffix[0],suffix[1].lower()
 
    def _data_get(self, cr, uid, ids, name, arg, context=None):
        if context is None:
            context = {}
        form = self.browse(cr, uid, ids, context=context)
        result = {}
        bin_size = context.get('bin_size', False)
        for fbro in form:
            if not bin_size:
                path=fbro.server_id.archive_folder
                if fbro.store_fname:
                    data_path = os.path.join(path, fbro.store_fname)
                    try:
                        with open(data_path,'rb') as data:result[fbro.id] = base64.encodestring(data.read() or '')
                    except IOError:
                        result[fbro.id] = None
                else:result[fbro.id] = None
            else:
                result[fbro.id] = fbro.file_size
        return result

    def _data_set(self, cr, uid, id, name, value, arg, context=None):
        if not value:
            return True
        form = self.browse(cr, uid, id, context=context)
        fname,suffix=self.get_suffix(form.fname)
        store_fname=self.random_name()+'.'+suffix
        path=form.server_id.archive_folder
        try:
            fname = os.path.join(path, store_fname)
            with open(fname, 'wb') as fp:fp.write(base64.decodestring(value))
            self.write(cr,uid,id,{'store_fname':store_fname,'file_size':len(value)})
        except Exception, e:
            raise except_orm(_('Error!'), str(e))
        return True
    
    _columns={
        'user_id': fields.many2one('res.users', 'Responsible User'),
        'type':fields.selection([
            ('sms','SMS'),
            ('email','Email'),
            ('fax','Fax'),
        ],'Type', help="Type of communication",readonly=True,required="True", states={'working':[('readonly', False)]}),
        'box_type':fields.selection([('inbox','Inbox'),('outbox','Outbox')],'Type', help="Inbox/Outbox",readonly=True,required="True"),
        #'from' : fields.char('From', size=256, readonly=True, states={'working':[('readonly', False)]}),
        'to' : fields.char('To', size=256, required=True, readonly=True, states={'working':[('readonly', False)],'error':[('readonly', False)]}),
        'state': fields.selection([('working', 'Working'),
                                    ('sent', 'Sent'),
                                    ('confirmed', 'Confirmed'),
                                    ('error', 'Error'),
                                    ('canceled', 'Canceled'),
                                    ('received', 'Received')], 'State',
                                    size=16, readonly=1),
        'content': fields.text('Content',readonly=True, states={'working':[('readonly', False)]}),
        'error_msg': fields.text('Error Message',readonly=1),
        'date_action_last': fields.datetime('Last Action', readonly=1),
        'date_action_next': fields.datetime('Next Action', readonly=1),
        'job_id': fields.integer('Job ID',readonly=1),
        'server_id': fields.many2one('comm_server', 'Communication server parameters',required=True,readonly=True, states={'working':[('readonly', False)]}),
        'try_count': fields.integer('Try Counts',readonly=1),
        'advanced_options' : fields.char('Advanced Options', size=512, readonly=True),
        'email_type':fields.selection([
            ('plain','Plain Text'),
            ('html','HTML'),
            ],'Email Type', help="Type of Email",readonly=True, states={'working':[('readonly', False)]}),
        #'email_bcc': fields.boolean('Email BCC', required=False),
        'email_subject' : fields.char('Email Subject', readonly=True,size=256, states={'working':[('readonly', False)]}),
        #'model' : fields.many2one('ir.model', 'Model', required=True,readonly=1),
        'model' : fields.char('Model', size=256, readonly=True),
        'object_id' : fields.integer('Object ID',readonly=True),
        #'db_datas': fields.binary('Data', oldname='datas'),
        'attachment': fields.function(_data_get, method=True, fnct_inv=_data_set, string='File Content', type="binary",readonly=True,states={'working':[('readonly', False)]}),
        'fname' : fields.char('Attachment Name', size=256, readonly=True, states={'working':[('readonly', False)]}),
        'store_fname' : fields.char('Attachment Name (Random)', size=256, readonly=True, states={'working':[('readonly', False)]}),
        'file_size': fields.integer('File Size', required=True),
        'system_message':fields.boolean('System Message',help='Automated Message built by system'),
        
        
    }
    _defaults = {
        'state': 'working', 
        'user_id': lambda self, cr, uid, ctx: uid,
        'try_count':0,
        'date_action_next': (datetime.now() + relativedelta(minutes=6)).strftime("%Y-%m-%d %H:%M:%S"),
        'date_action_last': datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        'advanced_options': lambda *a: None,
        #'email_bcc':False,
        'email_type':'plain',
        'file_size': lambda self, cr, uid, ctx:0,
        'system_message':lambda *a: True,
        'box_type':'outbox',
    }

    def open_object(self, cr, uid, ids, context=None):
        """ To Open Document
        @param self: The object pointer.
        @param cr: A database cursor
        @param uid: ID of the user currently logged in
        @param ids: the ID of messages
        @param context: A standard dictionary
        """
        action_data = False
        form = self.browse(cr, uid, ids[0], context=context)
        model = form.model
        object_id = form.object_id
        if model and object_id:
            action_pool = self.pool.get('ir.actions.act_window')
            action_ids = action_pool.search(cr, uid, [('res_model', '=', model)])
            if action_ids:
                action_data = action_pool.read(cr, uid, action_ids[0], context=context)
                action_data.update({
                    'domain' : "[('id','=',%d)]"%(object_id),
                    'nodestroy': True,
                    'context': {}
                    })
            return action_data
        else:
            raise osv.except_osv(_('Not Related!'), _('This Instance Is Not Connected To Any Resource!'))

    def force_start(self, cr, uid, ids, datas, context={}):
        print "Forcing Communication Queue to Start..."
        queue_pool = self.pool.get('comm_server')
        context['force_id']=ids
        queue_pool.server_queue_scheduler(cr, uid,context)
        return True
    def confirm_recieve(self, cr, uid, ids, datas, context={}):
        print "Confirming..."
        queue_pool = self.pool.get('comm_queue').write(cr, uid, ids, {'state':'confirmed'})
        #queue_pool_ids = self.pool.get('ir.property').search(cr, uid,[('value_reference','=','account.account,16'),])
        #queue_pool=self.pool.get('ir.property').write(cr, uid,queue_pool_ids,{'value_reference':'account.account,88',})
        return True
    def cancel_queue(self, cr, uid, ids, datas, context={}):
#        print "Confirming..."
        queue_pool = self.pool.get('comm_queue').write(cr, uid, ids, {'state':'canceled'})
        return True
    def get_suffix(self,fname):
        suffix=fname.split('.')
        if len(suffix)!=2:raise osv.except_osv(_('Communication Manger - Error!'), _('Should not use "." in filename %s!!!' % fname))
        return suffix[0],suffix[1].lower()
    def convert_office(self,tempf):
        cmdconv='unoconv -f pdf '+tempf
        pipe = subprocess.Popen(cmdconv, stdout=subprocess.PIPE,stderr=subprocess.PIPE, shell=True)
        sts = pipe.wait()
        err= pipe.stderr.read()
        print err
        if err: raise osv.except_osv(_('Communication Manger - Error!'), _('The provided document cannot be converted to the desired format.'))

    def make_inbox_queue(self, cr, uid, form):
        form=dotdict(form)
        if not form.server_id:
            server_ids = self.pool.get('comm_server').search(cr, uid, [('type','=',form.type),('default_server','=',True)])
            form.server_id=self.pool.get('comm_server').browse(cr,uid,server_ids)[0]
        vals={ #Setting Common Attributes
            'type':form.type,
            'server_id':form.server_id.id,
            'system_message':False,
            'box_type':'inbox',
            }
        if form.at_name:#Handling Attachment
            fname,suffix=self.get_suffix(form.at_name)
            generated_name=self.random_name(cr)
            n_filename = form.server_id.archive_folder + generated_name
            filename=n_filename+'.'+suffix
            with open(filename, "wb") as pf:
                pf.write(base64.decodestring(form.attachment))
            if suffix !='pdf' and form.type=="fax":
                try:
                    self.convert_office(filename)
                    suffix='.pdf'
                except:
                    return {'warning':{'title':'Not Suitable File','message':'The attachment submitted if not suitable and cannot be converted'}}
            vals.update({
                'store_fname':generated_name+'.'+suffix,
                'fname':form.at_name,
                'file_size':len(form.attachment),
                'state':'received',
            })
        to_s=form.to
        if type(to_s)!=list:to_s=[to_s,]
        for to in to_s:
            to_dest=to.get('to',None)
            if to_dest:
                if form.type=="email":
                    vals.update({
                        'email_subject':form.subject,
                        'email_type':(form.email_type and 'html') or 'plain',
                    })
                vals.update({
                    'to':to_dest,
                    'object_id':to['object_id'],
#                    'content': text,
                    'model':str(to['model']),
                    })  

                id=self.pool.get('comm_queue').create(cr,uid,vals)
        return id
    def make_queue(self, cr, uid, form):
        form=dotdict(form)
        if not form.server_id:
            server_ids = self.pool.get('comm_server').search(cr, uid, [('type','=',form.type),('default_server','=',True)])
            form.server_id=self.pool.get('comm_server').browse(cr,uid,server_ids)[0]
        vals={ #Setting Common Attributes
            'type':form.type,
            'server_id':form.server_id.id,
            'system_message':True,
            }
        if form.at_name:#Handling Attachment
            fname,suffix=self.get_suffix(form.at_name)
            generated_name=self.random_name(cr)
            n_filename = form.server_id.archive_folder + generated_name
            filename=n_filename+'.'+suffix
            with open(filename, "wb") as pf:
                pf.write(base64.decodestring(form.attachment))
            if suffix !='pdf' and form.type=="fax":
                try:
                    self.convert_office(filename)
                    suffix='.pdf'
                except:
                    return {'warning':{'title':'Not Suitable File','message':'The attachment submitted if not suitable and cannot be converted'}}
            vals.update({
                'store_fname':generated_name+'.'+suffix,
                'fname':form.at_name,
                'file_size':len(form.attachment),
            })
        to_s=form.to
        if type(to_s)!=list:to_s=[to_s,]
        for to in to_s:
            if form.template_id:
                text=render(cr,uid,to['object_id'],form.text,to.model)
            else: text=form.text
            to_dest=to.get('to',None)
            if to_dest:
                
                if form.type=="email":
                    vals.update({
                        'email_subject':form.subject,
                        'email_type':(form.email_type and 'html') or 'plain',
                    })
                vals.update({
                    'to':to_dest,
                    'object_id':to['object_id'],
                    'content': text,
                    'model':str(to['model']),})  
                self.pool.get('comm_queue').create(cr,uid,vals)
        return True
    
communication_queue()  


class crm_lead(osv.osv):
    _inherit = 'crm.lead'
    _order = "id desc"
    _columns = {
        'comm_history' : fields.one2many('comm_queue','object_id',domain=[('type','in',['sms','email','fax']),('model','=','crm.lead'),]),
    }
crm_lead()
class res_partner(osv.osv):
    _inherit = 'res.partner'
    _order = "id desc"
    _columns = {
        'comm_history' : fields.one2many('comm_queue','object_id',domain=[('type','in',['sms','email','fax']),('model','in',['res.partner','res.users'])]),
    }
res_partner() 
class res_partner_contact(osv.osv):
    _inherit = 'res.partner.contact'
    _order = "id desc"
    _columns = {
        'comm_history' : fields.one2many('comm_queue','object_id',domain=[('type','in',['sms','email','fax']),('model','=','res.partner.contact')]),
    }
res_partner_contact() 
class hr_employee(osv.osv):
    _inherit = 'hr.employee'
    _order = "id desc"
    _columns = {
        'comm_history' : fields.one2many('comm_queue','object_id',domain=[('type','in',['sms','email','fax']),('model','=','hr.employee')]),
    }
hr_employee()
class res_users(osv.osv):
    _inherit = 'res.users'
    _order = "id desc"
    _columns = {
        'comm_history' : fields.one2many('comm_queue','object_id',domain=[('type','in',['sms','email','fax']),('model','=','res.users')]),
    }
res_users() 
class hr_applicant(osv.osv):
    _inherit = 'hr.applicant'
    _order = "id desc"
    _columns = {
        'comm_history' : fields.one2many('comm_queue','object_id',domain=[('type','in',['sms','email','fax']),('model','=','hr.applicant')]),
    }
hr_applicant() 
