# -*- coding: utf-8 -*-
##############################################################################
#
#    Copyright (c) 2010-2012, Tashilgostar co. , All Rights Reserved
#        Nima ghorbani <nima.g.67@gmail.com> (http://ir.linkedin.com/in/nimaghi)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv,fields

import time
from datetime import datetime
from dateutil.relativedelta import relativedelta

from render import render

class comm_reminder(osv.osv):
    _name='comm_reminder'
    _order = "id desc"    
    _columns={
        'name' : fields.char('Name', size=256, required=True),
        'model': fields.many2one('ir.model','Model',required=True,),
        'comm_reminder_line_ids':fields.one2many('comm_reminder_line','reminder_id',domain=[('active','in',['True','False']),] ),
        'active':fields.boolean('Active',),
                }
    _defaults={
        'active': lambda *a : True,
        
        
        }
    def server_reminder_scheduler(self, cr, uid, context={}):
        print "In Reminder Scheduler..."
        vals={}
        reminder_pool = self.pool.get('comm_reminder')
        reminder_line_pool = self.pool.get('comm_reminder_line')
        
        reminder_line_items=reminder_line_pool.search(cr,uid,[('date_action_next','<=',datetime.now().strftime("%Y-%m-%d %H:%M:%S")),])
        reminder_items_ids=[line.reminder_id.id for line in reminder_line_pool.browse(cr,uid,reminder_line_items) ]
        if context.get('force_id',None):reminder_items_ids=context['force_id']
        print reminder_items_ids
        reminder_items=reminder_pool.browse(cr,uid,list(reminder_items_ids))
        for reminder in reminder_items:
            for reminder_line in reminder.comm_reminder_line_ids:
                if reminder_line.active:
                    comp_method=[(str(method.model_field.name),str(method.operator),eval(method.value)) for method in reminder_line.comp_method]
                    object_ids=self.pool.get(reminder.model.model).search(cr,uid,comp_method)
                    objects_pool=self.pool.get(reminder.model.model).browse(cr,uid,object_ids)
                    for object in objects_pool:
                        try:
                            dests=eval(reminder_line.to)
                        except:
                            continue
                        for dest in dests:
                            body,=render(cr,uid,object.id,reminder_line.template_id.body,reminder.model.model),
                            if reminder_line.type=='email':
                                vals.update({
                                    'subject':render(cr,uid,object.id,reminder_line.template_id.name,reminder.model.model),
                                    'email_type':(reminder_line.template_id.email_type and 'html') or 'plain',                                   
                                })
                            else:subject=None
                            to,model,object_id=dest
                            vals.update({
                                'text':body,
                                'type':reminder_line.type,
                                'server_id':None,
                                'at_name':None,
                                'attachment':None,
                                'system_message':True,
                                'to':[{'to':to,'model':model,'object_id':object_id},],
                            })
                            
                            self.pool.get('comm_queue').make_queue(cr, uid, vals)
                    next_type=reminder_line.interval_type
                    now=datetime.now()
                    if next_type=='minutes':next_t=(now + relativedelta(minutes= reminder_line.interval_number )).strftime("%Y-%m-%d %H:%M:%S")
                    elif  next_type=='hours':next_t=(now+ relativedelta(hours= reminder_line.interval_number )).strftime("%Y-%m-%d %H:%M:%S")
                    elif next_type=='days':next_t=(now + relativedelta(days= reminder_line.interval_number )).strftime("%Y-%m-%d %H:%M:%S")
                    else :next_t=(now+ relativedelta(months= reminder_line.interval_number )).strftime("%Y-%m-%d %H:%M:%S")
                    
                    self.pool.get('comm_reminder_line').write(cr,uid,reminder_line.id,{'date_action_next': next_t,'date_action_last':now.strftime("%Y-%m-%d %H:%M:%S")})
                    

        return True
    
    def test(self, cr, uid, ids, datas, context={}):
        reminder_pool = self.pool.get('comm_reminder')
        context['force_id']=ids
        reminder_pool.server_reminder_scheduler(cr, uid,context)
        return True
comm_reminder()


class comm_reminder_line(osv.osv):
    _name='comm_reminder_line'
    _order = "id desc"
    _columns={
        'name' : fields.char('Name', size=256, required=True),
        'reminder_id':fields.many2one('comm_reminder','Reminder'),
        'type':fields.selection([
            ('sms','SMS'),
            ('email','Email'),
        ],'Type', help="Type of Communication",readonly=False,required="True"),
        'template_id': fields.many2one('communication.templates','Template'),
        'to' : fields.char('To', size=256, required=True,help="[(Destination Address,Model,object_id),]"),
        #'comp_method' : fields.char('Comparison Method', size=1024, required=True, ),
        'comp_method':fields.one2many('comm_reminder_comparison','comm_reminder_line_id',domain=[('active','in',['True','False']),] ),
        'active':fields.boolean('Active',),
        'date_action_last': fields.datetime('Last Action', readonly=1),
        'date_action_next': fields.datetime('Next Action', readonly=1),
        'interval_number': fields.integer('Interval Number',help="Repeat every x."),
        'interval_type': fields.selection( [('minutes', 'Minutes'),('hours', 'Hours'), ('days', 'Days'),('months', 'Months')], 'Interval Unit'),
        }
    _defaults={
        'active': lambda *a : True,
        'comp_method': lambda *a : None,
        'date_action_last': datetime.now().strftime("%Y-%m-%d 00:00:00"),
        'date_action_next': datetime.now().strftime("%Y-%m-%d 00:00:00"),
        'interval_number':1,
        'interval_type':'days',
        }
    
comm_reminder_line()

class comm_reminder_comparison(osv.osv):
    _name='comm_reminder_comparison'
    _order = "id desc"
    _columns={
        #'model_field': fields.many2one('ir.model.fields','Field',required=True,domain="[('model_id','=',comm_reminder_line_id.id)]"),
        'model_field': fields.many2one('ir.model.fields','Field',required=True,),        
        'comm_reminder_line_id':fields.many2one('comm_reminder_line','Reminder Line',required=True),
        'operator':fields.selection([
            ('=','=='),
            ('<=','<='),
            ('>=','>='),
            ('!=','!='),
            ('in','in'),
            ('not in','not in'),
        ],'Method', help="Method Of Comparison",readonly=False,required=True),
        'value' : fields.char('Value', size=512, required=True),
        'active':fields.boolean('Active',),
        }

    _defaults={
        'active': lambda *a : True,
       # 'comm_reminder_line_id': _nima_test,
        }

comm_reminder_comparison()

